# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2
from feast.protos.feast.core import Entity_pb2 as feast_dot_core_dot_Entity__pb2
from feast.protos.feast.core import FeatureService_pb2 as feast_dot_core_dot_FeatureService__pb2
from feast.protos.feast.core import FeatureView_pb2 as feast_dot_core_dot_FeatureView__pb2
from feast.protos.feast.core import InfraObject_pb2 as feast_dot_core_dot_InfraObject__pb2
from feast.protos.feast.core import OnDemandFeatureView_pb2 as feast_dot_core_dot_OnDemandFeatureView__pb2
from feast.protos.feast.core import Permission_pb2 as feast_dot_core_dot_Permission__pb2
from feast.protos.feast.core import Project_pb2 as feast_dot_core_dot_Project__pb2
from feast.protos.feast.core import Registry_pb2 as feast_dot_core_dot_Registry__pb2
from feast.protos.feast.core import SavedDataset_pb2 as feast_dot_core_dot_SavedDataset__pb2
from feast.protos.feast.core import StreamFeatureView_pb2 as feast_dot_core_dot_StreamFeatureView__pb2
from feast.protos.feast.core import ValidationProfile_pb2 as feast_dot_core_dot_ValidationProfile__pb2
from feast.protos.feast.registry import RegistryServer_pb2 as feast_dot_registry_dot_RegistryServer__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


class RegistryServerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ApplyEntity = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyEntity',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyEntityRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetEntity = channel.unary_unary(
                '/feast.registry.RegistryServer/GetEntity',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetEntityRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_Entity__pb2.Entity.FromString,
                )
        self.ListEntities = channel.unary_unary(
                '/feast.registry.RegistryServer/ListEntities',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesResponse.FromString,
                )
        self.DeleteEntity = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteEntity',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteEntityRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyDataSource = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyDataSource',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyDataSourceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetDataSource = channel.unary_unary(
                '/feast.registry.RegistryServer/GetDataSource',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetDataSourceRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_DataSource__pb2.DataSource.FromString,
                )
        self.ListDataSources = channel.unary_unary(
                '/feast.registry.RegistryServer/ListDataSources',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesResponse.FromString,
                )
        self.DeleteDataSource = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteDataSource',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteDataSourceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureViewRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.DeleteFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureViewRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetAnyFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/GetAnyFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewResponse.FromString,
                )
        self.ListAllFeatureViews = channel.unary_unary(
                '/feast.registry.RegistryServer/ListAllFeatureViews',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsResponse.FromString,
                )
        self.GetFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/GetFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureViewRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_FeatureView__pb2.FeatureView.FromString,
                )
        self.ListFeatureViews = channel.unary_unary(
                '/feast.registry.RegistryServer/ListFeatureViews',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsResponse.FromString,
                )
        self.GetStreamFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/GetStreamFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetStreamFeatureViewRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_StreamFeatureView__pb2.StreamFeatureView.FromString,
                )
        self.ListStreamFeatureViews = channel.unary_unary(
                '/feast.registry.RegistryServer/ListStreamFeatureViews',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsResponse.FromString,
                )
        self.GetOnDemandFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/GetOnDemandFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetOnDemandFeatureViewRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_OnDemandFeatureView__pb2.OnDemandFeatureView.FromString,
                )
        self.ListOnDemandFeatureViews = channel.unary_unary(
                '/feast.registry.RegistryServer/ListOnDemandFeatureViews',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsResponse.FromString,
                )
        self.ApplyFeatureService = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyFeatureService',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureServiceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetFeatureService = channel.unary_unary(
                '/feast.registry.RegistryServer/GetFeatureService',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureServiceRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_FeatureService__pb2.FeatureService.FromString,
                )
        self.ListFeatureServices = channel.unary_unary(
                '/feast.registry.RegistryServer/ListFeatureServices',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesResponse.FromString,
                )
        self.DeleteFeatureService = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteFeatureService',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureServiceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplySavedDataset = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplySavedDataset',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplySavedDatasetRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetSavedDataset = channel.unary_unary(
                '/feast.registry.RegistryServer/GetSavedDataset',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetSavedDatasetRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_SavedDataset__pb2.SavedDataset.FromString,
                )
        self.ListSavedDatasets = channel.unary_unary(
                '/feast.registry.RegistryServer/ListSavedDatasets',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsResponse.FromString,
                )
        self.DeleteSavedDataset = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteSavedDataset',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteSavedDatasetRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyValidationReference = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyValidationReference',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyValidationReferenceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetValidationReference = channel.unary_unary(
                '/feast.registry.RegistryServer/GetValidationReference',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetValidationReferenceRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_ValidationProfile__pb2.ValidationReference.FromString,
                )
        self.ListValidationReferences = channel.unary_unary(
                '/feast.registry.RegistryServer/ListValidationReferences',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesResponse.FromString,
                )
        self.DeleteValidationReference = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteValidationReference',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteValidationReferenceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyPermission = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyPermission',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyPermissionRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetPermission = channel.unary_unary(
                '/feast.registry.RegistryServer/GetPermission',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetPermissionRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_Permission__pb2.Permission.FromString,
                )
        self.ListPermissions = channel.unary_unary(
                '/feast.registry.RegistryServer/ListPermissions',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsResponse.FromString,
                )
        self.DeletePermission = channel.unary_unary(
                '/feast.registry.RegistryServer/DeletePermission',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeletePermissionRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyProject = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyProject',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyProjectRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetProject = channel.unary_unary(
                '/feast.registry.RegistryServer/GetProject',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetProjectRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_Project__pb2.Project.FromString,
                )
        self.ListProjects = channel.unary_unary(
                '/feast.registry.RegistryServer/ListProjects',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectsResponse.FromString,
                )
        self.DeleteProject = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteProject',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteProjectRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyMaterialization = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyMaterialization',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyMaterializationRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ListProjectMetadata = channel.unary_unary(
                '/feast.registry.RegistryServer/ListProjectMetadata',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataResponse.FromString,
                )
        self.UpdateInfra = channel.unary_unary(
                '/feast.registry.RegistryServer/UpdateInfra',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.UpdateInfraRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetInfra = channel.unary_unary(
                '/feast.registry.RegistryServer/GetInfra',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetInfraRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_InfraObject__pb2.Infra.FromString,
                )
        self.Commit = channel.unary_unary(
                '/feast.registry.RegistryServer/Commit',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.Refresh = channel.unary_unary(
                '/feast.registry.RegistryServer/Refresh',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.RefreshRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.Proto = channel.unary_unary(
                '/feast.registry.RegistryServer/Proto',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=feast_dot_core_dot_Registry__pb2.Registry.FromString,
                )
        self.GetRegistryLineage = channel.unary_unary(
                '/feast.registry.RegistryServer/GetRegistryLineage',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageResponse.FromString,
                )
        self.GetObjectRelationships = channel.unary_unary(
                '/feast.registry.RegistryServer/GetObjectRelationships',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsResponse.FromString,
                )
        self.ListFeatures = channel.unary_unary(
                '/feast.registry.RegistryServer/ListFeatures',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesResponse.FromString,
                )
        self.GetFeature = channel.unary_unary(
                '/feast.registry.RegistryServer/GetFeature',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.Feature.FromString,
                )


class RegistryServerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ApplyEntity(self, request, context):
        """Entity RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetEntity(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListEntities(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteEntity(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyDataSource(self, request, context):
        """DataSource RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDataSource(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListDataSources(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteDataSource(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyFeatureView(self, request, context):
        """FeatureView RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteFeatureView(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAnyFeatureView(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListAllFeatureViews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeatureView(self, request, context):
        """plain FeatureView RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatureViews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStreamFeatureView(self, request, context):
        """StreamFeatureView RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListStreamFeatureViews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetOnDemandFeatureView(self, request, context):
        """OnDemandFeatureView RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListOnDemandFeatureViews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyFeatureService(self, request, context):
        """FeatureService RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeatureService(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatureServices(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteFeatureService(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplySavedDataset(self, request, context):
        """SavedDataset RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSavedDataset(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSavedDatasets(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSavedDataset(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyValidationReference(self, request, context):
        """ValidationReference RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetValidationReference(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListValidationReferences(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteValidationReference(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyPermission(self, request, context):
        """Permission RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPermissions(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeletePermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyProject(self, request, context):
        """Project RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetProject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListProjects(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteProject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyMaterialization(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListProjectMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateInfra(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInfra(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Commit(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Refresh(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Proto(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRegistryLineage(self, request, context):
        """Lineage RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetObjectRelationships(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatures(self, request, context):
        """Feature RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeature(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RegistryServerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ApplyEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyEntity,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyEntityRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEntity,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetEntityRequest.FromString,
                    response_serializer=feast_dot_core_dot_Entity__pb2.Entity.SerializeToString,
            ),
            'ListEntities': grpc.unary_unary_rpc_method_handler(
                    servicer.ListEntities,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesResponse.SerializeToString,
            ),
            'DeleteEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteEntity,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteEntityRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyDataSource': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyDataSource,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyDataSourceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetDataSource': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDataSource,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetDataSourceRequest.FromString,
                    response_serializer=feast_dot_core_dot_DataSource__pb2.DataSource.SerializeToString,
            ),
            'ListDataSources': grpc.unary_unary_rpc_method_handler(
                    servicer.ListDataSources,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesResponse.SerializeToString,
            ),
            'DeleteDataSource': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteDataSource,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteDataSourceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureViewRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'DeleteFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureViewRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetAnyFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAnyFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewResponse.SerializeToString,
            ),
            'ListAllFeatureViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListAllFeatureViews,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsResponse.SerializeToString,
            ),
            'GetFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureViewRequest.FromString,
                    response_serializer=feast_dot_core_dot_FeatureView__pb2.FeatureView.SerializeToString,
            ),
            'ListFeatureViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFeatureViews,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsResponse.SerializeToString,
            ),
            'GetStreamFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStreamFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetStreamFeatureViewRequest.FromString,
                    response_serializer=feast_dot_core_dot_StreamFeatureView__pb2.StreamFeatureView.SerializeToString,
            ),
            'ListStreamFeatureViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListStreamFeatureViews,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsResponse.SerializeToString,
            ),
            'GetOnDemandFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetOnDemandFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetOnDemandFeatureViewRequest.FromString,
                    response_serializer=feast_dot_core_dot_OnDemandFeatureView__pb2.OnDemandFeatureView.SerializeToString,
            ),
            'ListOnDemandFeatureViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListOnDemandFeatureViews,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsResponse.SerializeToString,
            ),
            'ApplyFeatureService': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyFeatureService,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureServiceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetFeatureService': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeatureService,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureServiceRequest.FromString,
                    response_serializer=feast_dot_core_dot_FeatureService__pb2.FeatureService.SerializeToString,
            ),
            'ListFeatureServices': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFeatureServices,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesResponse.SerializeToString,
            ),
            'DeleteFeatureService': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteFeatureService,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureServiceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplySavedDataset': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplySavedDataset,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplySavedDatasetRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetSavedDataset': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSavedDataset,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetSavedDatasetRequest.FromString,
                    response_serializer=feast_dot_core_dot_SavedDataset__pb2.SavedDataset.SerializeToString,
            ),
            'ListSavedDatasets': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSavedDatasets,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsResponse.SerializeToString,
            ),
            'DeleteSavedDataset': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteSavedDataset,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteSavedDatasetRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyValidationReference': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyValidationReference,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyValidationReferenceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetValidationReference': grpc.unary_unary_rpc_method_handler(
                    servicer.GetValidationReference,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetValidationReferenceRequest.FromString,
                    response_serializer=feast_dot_core_dot_ValidationProfile__pb2.ValidationReference.SerializeToString,
            ),
            'ListValidationReferences': grpc.unary_unary_rpc_method_handler(
                    servicer.ListValidationReferences,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesResponse.SerializeToString,
            ),
            'DeleteValidationReference': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteValidationReference,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteValidationReferenceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyPermission,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyPermissionRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPermission,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetPermissionRequest.FromString,
                    response_serializer=feast_dot_core_dot_Permission__pb2.Permission.SerializeToString,
            ),
            'ListPermissions': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPermissions,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsResponse.SerializeToString,
            ),
            'DeletePermission': grpc.unary_unary_rpc_method_handler(
                    servicer.DeletePermission,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeletePermissionRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyProject': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyProject,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyProjectRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetProject': grpc.unary_unary_rpc_method_handler(
                    servicer.GetProject,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetProjectRequest.FromString,
                    response_serializer=feast_dot_core_dot_Project__pb2.Project.SerializeToString,
            ),
            'ListProjects': grpc.unary_unary_rpc_method_handler(
                    servicer.ListProjects,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectsResponse.SerializeToString,
            ),
            'DeleteProject': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteProject,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteProjectRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyMaterialization': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyMaterialization,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyMaterializationRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ListProjectMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.ListProjectMetadata,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataResponse.SerializeToString,
            ),
            'UpdateInfra': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateInfra,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.UpdateInfraRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetInfra': grpc.unary_unary_rpc_method_handler(
                    servicer.GetInfra,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetInfraRequest.FromString,
                    response_serializer=feast_dot_core_dot_InfraObject__pb2.Infra.SerializeToString,
            ),
            'Commit': grpc.unary_unary_rpc_method_handler(
                    servicer.Commit,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'Refresh': grpc.unary_unary_rpc_method_handler(
                    servicer.Refresh,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.RefreshRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'Proto': grpc.unary_unary_rpc_method_handler(
                    servicer.Proto,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=feast_dot_core_dot_Registry__pb2.Registry.SerializeToString,
            ),
            'GetRegistryLineage': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRegistryLineage,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageResponse.SerializeToString,
            ),
            'GetObjectRelationships': grpc.unary_unary_rpc_method_handler(
                    servicer.GetObjectRelationships,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsResponse.SerializeToString,
            ),
            'ListFeatures': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFeatures,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesResponse.SerializeToString,
            ),
            'GetFeature': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeature,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.Feature.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'feast.registry.RegistryServer', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class RegistryServer(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ApplyEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyEntity',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyEntityRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetEntity',
            feast_dot_registry_dot_RegistryServer__pb2.GetEntityRequest.SerializeToString,
            feast_dot_core_dot_Entity__pb2.Entity.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListEntities(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListEntities',
            feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteEntity',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteEntityRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyDataSource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyDataSource',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyDataSourceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDataSource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetDataSource',
            feast_dot_registry_dot_RegistryServer__pb2.GetDataSourceRequest.SerializeToString,
            feast_dot_core_dot_DataSource__pb2.DataSource.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListDataSources(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListDataSources',
            feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteDataSource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteDataSource',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteDataSourceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureViewRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureViewRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAnyFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetAnyFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListAllFeatureViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListAllFeatureViews',
            feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.GetFeatureViewRequest.SerializeToString,
            feast_dot_core_dot_FeatureView__pb2.FeatureView.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatureViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListFeatureViews',
            feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStreamFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetStreamFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.GetStreamFeatureViewRequest.SerializeToString,
            feast_dot_core_dot_StreamFeatureView__pb2.StreamFeatureView.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListStreamFeatureViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListStreamFeatureViews',
            feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetOnDemandFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetOnDemandFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.GetOnDemandFeatureViewRequest.SerializeToString,
            feast_dot_core_dot_OnDemandFeatureView__pb2.OnDemandFeatureView.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListOnDemandFeatureViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListOnDemandFeatureViews',
            feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyFeatureService(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyFeatureService',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureServiceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeatureService(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetFeatureService',
            feast_dot_registry_dot_RegistryServer__pb2.GetFeatureServiceRequest.SerializeToString,
            feast_dot_core_dot_FeatureService__pb2.FeatureService.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatureServices(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListFeatureServices',
            feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteFeatureService(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteFeatureService',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureServiceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplySavedDataset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplySavedDataset',
            feast_dot_registry_dot_RegistryServer__pb2.ApplySavedDatasetRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSavedDataset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetSavedDataset',
            feast_dot_registry_dot_RegistryServer__pb2.GetSavedDatasetRequest.SerializeToString,
            feast_dot_core_dot_SavedDataset__pb2.SavedDataset.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListSavedDatasets(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListSavedDatasets',
            feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteSavedDataset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteSavedDataset',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteSavedDatasetRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyValidationReference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyValidationReference',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyValidationReferenceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetValidationReference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetValidationReference',
            feast_dot_registry_dot_RegistryServer__pb2.GetValidationReferenceRequest.SerializeToString,
            feast_dot_core_dot_ValidationProfile__pb2.ValidationReference.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListValidationReferences(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListValidationReferences',
            feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteValidationReference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteValidationReference',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteValidationReferenceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyPermission',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyPermissionRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetPermission',
            feast_dot_registry_dot_RegistryServer__pb2.GetPermissionRequest.SerializeToString,
            feast_dot_core_dot_Permission__pb2.Permission.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListPermissions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListPermissions',
            feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeletePermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeletePermission',
            feast_dot_registry_dot_RegistryServer__pb2.DeletePermissionRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyProject',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyProjectRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetProject',
            feast_dot_registry_dot_RegistryServer__pb2.GetProjectRequest.SerializeToString,
            feast_dot_core_dot_Project__pb2.Project.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListProjects(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListProjects',
            feast_dot_registry_dot_RegistryServer__pb2.ListProjectsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListProjectsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteProject',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteProjectRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyMaterialization(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyMaterialization',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyMaterializationRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListProjectMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListProjectMetadata',
            feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateInfra(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/UpdateInfra',
            feast_dot_registry_dot_RegistryServer__pb2.UpdateInfraRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetInfra(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetInfra',
            feast_dot_registry_dot_RegistryServer__pb2.GetInfraRequest.SerializeToString,
            feast_dot_core_dot_InfraObject__pb2.Infra.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Commit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/Commit',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Refresh(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/Refresh',
            feast_dot_registry_dot_RegistryServer__pb2.RefreshRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Proto(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/Proto',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            feast_dot_core_dot_Registry__pb2.Registry.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRegistryLineage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetRegistryLineage',
            feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetObjectRelationships(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetObjectRelationships',
            feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatures(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListFeatures',
            feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeature(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetFeature',
            feast_dot_registry_dot_RegistryServer__pb2.GetFeatureRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.Feature.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from typing import Union, get_args

from feast.project import Project
from feast.protos.feast.core.Project_pb2 import ProjectSpec

from .batch_feature_view import BatchFeatureView
from .data_source import DataSource
from .entity import Entity
from .feature_service import FeatureService
from .feature_view import FeatureView
from .on_demand_feature_view import OnDemandFeatureView
from .permissions.permission import Permission
from .protos.feast.core.DataSource_pb2 import DataSource as DataSourceProto
from .protos.feast.core.Entity_pb2 import EntitySpecV2
from .protos.feast.core.FeatureService_pb2 import FeatureServiceSpec
from .protos.feast.core.FeatureView_pb2 import FeatureViewSpec
from .protos.feast.core.OnDemandFeatureView_pb2 import OnDemandFeatureViewSpec
from .protos.feast.core.Permission_pb2 import PermissionSpec as PermissionSpec
from .protos.feast.core.SavedDataset_pb2 import SavedDatasetSpec
from .protos.feast.core.StreamFeatureView_pb2 import StreamFeatureViewSpec
from .protos.feast.core.ValidationProfile_pb2 import (
    ValidationReference as ValidationReferenceProto,
)
from .saved_dataset import SavedDataset, ValidationReference
from .stream_feature_view import StreamFeatureView

# Convenience type representing all Feast objects
FeastObject = Union[
    Project,
    FeatureView,
    OnDemandFeatureView,
    BatchFeatureView,
    StreamFeatureView,
    Entity,
    FeatureService,
    DataSource,
    ValidationReference,
    SavedDataset,
    Permission,
]

FeastObjectSpecProto = Union[
    ProjectSpec,
    FeatureViewSpec,
    OnDemandFeatureViewSpec,
    StreamFeatureViewSpec,
    EntitySpecV2,
    FeatureServiceSpec,
    DataSourceProto,
    ValidationReferenceProto,
    SavedDatasetSpec,
    PermissionSpec,
]

ALL_RESOURCE_TYPES = list(get_args(FeastObject))
ALL_FEATURE_VIEW_TYPES = [
    FeatureView,
    OnDemandFeatureView,
    BatchFeatureView,
    StreamFeatureView,
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file

* Copyright 2020 The Feast Authors
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     https://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
"""
import builtins
import collections.abc
import feast.types.Value_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Entity(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SPEC_FIELD_NUMBER: builtins.int
    META_FIELD_NUMBER: builtins.int
    @property
    def spec(self) -> global___EntitySpecV2:
        """User-specified specifications of this entity."""
    @property
    def meta(self) -> global___EntityMeta:
        """System-populated metadata for this entity."""
    def __init__(
        self,
        *,
        spec: global___EntitySpecV2 | None = ...,
        meta: global___EntityMeta | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["meta", b"meta", "spec", b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["meta", b"meta", "spec", b"spec"]) -> None: ...

global___Entity = Entity

class EntitySpecV2(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    VALUE_TYPE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    JOIN_KEY_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the entity."""
    project: builtins.str
    """Name of Feast project that this feature table belongs to."""
    value_type: feast.types.Value_pb2.ValueType.Enum.ValueType
    """Type of the entity."""
    description: builtins.str
    """Description of the entity."""
    join_key: builtins.str
    """Join key for the entity (i.e. name of the column the entity maps to)."""
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """User defined metadata"""
    owner: builtins.str
    """Owner of the entity."""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        value_type: feast.types.Value_pb2.ValueType.Enum.ValueType = ...,
        description: builtins.str = ...,
        join_key: builtins.str = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        owner: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description", "join_key", b"join_key", "name", b"name", "owner", b"owner", "project", b"project", "tags", b"tags", "value_type", b"value_type"]) -> None: ...

global___EntitySpecV2 = EntitySpecV2

class EntityMeta(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATED_TIMESTAMP_FIELD_NUMBER: builtins.int
    LAST_UPDATED_TIMESTAMP_FIELD_NUMBER: builtins.int
    @property
    def created_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def last_updated_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        created_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        last_updated_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["created_timestamp", b"created_timestamp", "last_updated_timestamp", b"last_updated_timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["created_timestamp", b"created_timestamp", "last_updated_timestamp", b"last_updated_timestamp"]) -> None: ...

global___EntityMeta = EntityMeta

class EntityList(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENTITIES_FIELD_NUMBER: builtins.int
    @property
    def entities(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Entity]: ...
    def __init__(
        self,
        *,
        entities: collections.abc.Iterable[global___Entity] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["entities", b"entities"]) -> None: ...

global___EntityList = EntityList



from fastapi import APIRouter, Depends, Query

from feast.api.registry.rest.rest_utils import (
    get_pagination_params,
    get_sorting_params,
    grpc_call,
    list_all_projects,
)
from feast.protos.feast.registry import RegistryServer_pb2


def get_project_router(grpc_handler) -> APIRouter:
    router = APIRouter()

    @router.get("/projects/{name}")
    def get_project(
        name: str,
        allow_cache: bool = Query(True),
    ):
        req = RegistryServer_pb2.GetProjectRequest(
            name=name,
            allow_cache=allow_cache,
        )
        project = grpc_call(grpc_handler.GetProject, req)

        return project

    @router.get("/projects")
    def list_projects(
        allow_cache: bool = Query(True),
        pagination_params: dict = Depends(get_pagination_params),
        sorting_params: dict = Depends(get_sorting_params),
    ):
        try:
            projects, pagination, err_msg = list_all_projects(
                grpc_handler=grpc_handler,
                allow_cache=allow_cache,
                pagination_params=pagination_params,
                sorting_params=sorting_params,
            )
        except Exception as e:
            return {"error": str(e)}

        if err_msg:
            return {"error": err_msg}

        return {
            "projects": projects,
            "pagination": pagination,
        }

    return router



from abc import ABC
from typing import List, cast

from feast.infra.compute_engines.dag.context import ExecutionContext
from feast.infra.compute_engines.dag.node import DAGNode
from feast.infra.compute_engines.local.arrow_table_value import ArrowTableValue


class LocalNode(DAGNode, ABC):
    def get_single_table(self, context: ExecutionContext) -> ArrowTableValue:
        return cast(ArrowTableValue, self.get_single_input_value(context))

    def get_input_tables(self, context: ExecutionContext) -> List[ArrowTableValue]:
        return [cast(ArrowTableValue, val) for val in self.get_input_values(context)]



"""
A module with utility functions to support the authorization management in Feast servers.
"""

import enum
import logging

import feast
from feast.permissions.auth.auth_manager import (
    AllowAll,
    AuthManager,
    set_auth_manager,
)
from feast.permissions.auth.oidc_token_parser import OidcTokenParser
from feast.permissions.auth.token_extractor import TokenExtractor
from feast.permissions.auth.token_parser import TokenParser
from feast.permissions.auth_model import (
    AuthConfig,
    OidcAuthConfig,
)
from feast.permissions.security_manager import (
    SecurityManager,
    no_security_manager,
    set_security_manager,
)
from feast.permissions.server.arrow_flight_token_extractor import (
    ArrowFlightTokenExtractor,
)
from feast.permissions.server.grpc_token_extractor import GrpcTokenExtractor
from feast.permissions.server.rest_token_extractor import RestTokenExtractor

logger = logging.getLogger(__name__)


class ServerType(enum.Enum):
    """
    Identify the server type.
    """

    REST = "rest"
    ARROW = "arrow"
    GRPC = "grpc"  # TODO RBAC: to be completed


class AuthManagerType(enum.Enum):
    """
    Identify the type of authorization manager.
    """

    NONE = "no_auth"
    OIDC = "oidc"
    KUBERNETES = "kubernetes"


def str_to_auth_manager_type(value: str) -> AuthManagerType:
    for t in AuthManagerType.__members__.values():
        if t.value.lower() == value.lower():
            return t

    logger.warning(
        f"Requested unmanaged AuthManagerType of value {value}. Using NONE instead."
    )
    return AuthManagerType.NONE


def init_security_manager(auth_type: AuthManagerType, fs: "feast.FeatureStore"):
    """
    Initialize the global security manager.
    Must be invoked at Feast server initialization time to create the `SecurityManager` instance.

    Args:
        auth_type: The authorization manager type.
        registry: The feature store registry.
    """
    if auth_type == AuthManagerType.NONE:
        no_security_manager()
    else:
        # TODO permissions from registry
        set_security_manager(
            SecurityManager(
                project=fs.project,
                registry=fs.registry,
            )
        )


def init_auth_manager(
    server_type: ServerType, auth_type: AuthManagerType, auth_config: AuthConfig
):
    """
    Initialize the global authorization manager.
    Must be invoked at Feast server initialization time to create the `AuthManager` instance.

    Args:
        server_type: The server type.
        auth_type: The authorization manager type.

    Raises:
        ValueError: If any input argument has an unmanaged value.
    """
    if auth_type == AuthManagerType.NONE:
        set_auth_manager(AllowAll())
    else:
        token_extractor: TokenExtractor
        token_parser: TokenParser

        if server_type == ServerType.REST:
            token_extractor = RestTokenExtractor()
        elif server_type == ServerType.ARROW:
            token_extractor = ArrowFlightTokenExtractor()
        elif server_type == ServerType.GRPC:
            token_extractor = GrpcTokenExtractor()
        else:
            raise ValueError(f"Unmanaged server type {server_type}")

        if auth_type == AuthManagerType.KUBERNETES:
            from feast.permissions.auth.kubernetes_token_parser import (
                KubernetesTokenParser,
            )

            token_parser = KubernetesTokenParser()
        elif auth_type == AuthManagerType.OIDC:
            assert isinstance(auth_config, OidcAuthConfig)
            token_parser = OidcTokenParser(auth_config=auth_config)
        else:
            raise ValueError(f"Unmanaged authorization manager type {auth_type}")

        auth_manager = AuthManager(
            token_extractor=token_extractor, token_parser=token_parser
        )
        set_auth_manager(auth_manager)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/StreamFeatureView.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from feast.protos.feast.core import OnDemandFeatureView_pb2 as feast_dot_core_dot_OnDemandFeatureView__pb2
from feast.protos.feast.core import FeatureView_pb2 as feast_dot_core_dot_FeatureView__pb2
from feast.protos.feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2
from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2
from feast.protos.feast.core import Aggregation_pb2 as feast_dot_core_dot_Aggregation__pb2
from feast.protos.feast.core import Transformation_pb2 as feast_dot_core_dot_Transformation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"feast/core/StreamFeatureView.proto\x12\nfeast.core\x1a\x1egoogle/protobuf/duration.proto\x1a$feast/core/OnDemandFeatureView.proto\x1a\x1c\x66\x65\x61st/core/FeatureView.proto\x1a\x18\x66\x65\x61st/core/Feature.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\x1a\x1c\x66\x65\x61st/core/Aggregation.proto\x1a\x1f\x66\x65\x61st/core/Transformation.proto\"o\n\x11StreamFeatureView\x12/\n\x04spec\x18\x01 \x01(\x0b\x32!.feast.core.StreamFeatureViewSpec\x12)\n\x04meta\x18\x02 \x01(\x0b\x32\x1b.feast.core.FeatureViewMeta\"\xa8\x05\n\x15StreamFeatureViewSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x10\n\x08\x65ntities\x18\x03 \x03(\t\x12+\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32\x19.feast.core.FeatureSpecV2\x12\x31\n\x0e\x65ntity_columns\x18\x05 \x03(\x0b\x32\x19.feast.core.FeatureSpecV2\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12\x39\n\x04tags\x18\x07 \x03(\x0b\x32+.feast.core.StreamFeatureViewSpec.TagsEntry\x12\r\n\x05owner\x18\x08 \x01(\t\x12&\n\x03ttl\x18\t \x01(\x0b\x32\x19.google.protobuf.Duration\x12,\n\x0c\x62\x61tch_source\x18\n \x01(\x0b\x32\x16.feast.core.DataSource\x12-\n\rstream_source\x18\x0b \x01(\x0b\x32\x16.feast.core.DataSource\x12\x0e\n\x06online\x18\x0c \x01(\x08\x12\x42\n\x15user_defined_function\x18\r \x01(\x0b\x32\x1f.feast.core.UserDefinedFunctionB\x02\x18\x01\x12\x0c\n\x04mode\x18\x0e \x01(\t\x12-\n\x0c\x61ggregations\x18\x0f \x03(\x0b\x32\x17.feast.core.Aggregation\x12\x17\n\x0ftimestamp_field\x18\x10 \x01(\t\x12\x43\n\x16\x66\x65\x61ture_transformation\x18\x11 \x01(\x0b\x32#.feast.core.FeatureTransformationV2\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42[\n\x10\x66\x65\x61st.proto.coreB\x16StreamFeatureViewProtoZ/github.com/feast-dev/feast/go/protos/feast/coreb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.StreamFeatureView_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\020feast.proto.coreB\026StreamFeatureViewProtoZ/github.com/feast-dev/feast/go/protos/feast/core'
  _globals['_STREAMFEATUREVIEWSPEC_TAGSENTRY']._options = None
  _globals['_STREAMFEATUREVIEWSPEC_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_STREAMFEATUREVIEWSPEC'].fields_by_name['user_defined_function']._options = None
  _globals['_STREAMFEATUREVIEWSPEC'].fields_by_name['user_defined_function']._serialized_options = b'\030\001'
  _globals['_STREAMFEATUREVIEW']._serialized_start=268
  _globals['_STREAMFEATUREVIEW']._serialized_end=379
  _globals['_STREAMFEATUREVIEWSPEC']._serialized_start=382
  _globals['_STREAMFEATUREVIEWSPEC']._serialized_end=1062
  _globals['_STREAMFEATUREVIEWSPEC_TAGSENTRY']._serialized_start=1019
  _globals['_STREAMFEATUREVIEWSPEC_TAGSENTRY']._serialized_end=1062
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class UserDefinedFunctionV2(google.protobuf.message.Message):
    """Serialized representation of python function."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    BODY_FIELD_NUMBER: builtins.int
    BODY_TEXT_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The function name"""
    body: builtins.bytes
    """The python-syntax function body (serialized by dill)"""
    body_text: builtins.str
    """The string representation of the udf"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        body: builtins.bytes = ...,
        body_text: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["body", b"body", "body_text", b"body_text", "name", b"name"]) -> None: ...

global___UserDefinedFunctionV2 = UserDefinedFunctionV2

class FeatureTransformationV2(google.protobuf.message.Message):
    """A feature transformation executed as a user-defined function"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USER_DEFINED_FUNCTION_FIELD_NUMBER: builtins.int
    SUBSTRAIT_TRANSFORMATION_FIELD_NUMBER: builtins.int
    @property
    def user_defined_function(self) -> global___UserDefinedFunctionV2: ...
    @property
    def substrait_transformation(self) -> global___SubstraitTransformationV2: ...
    def __init__(
        self,
        *,
        user_defined_function: global___UserDefinedFunctionV2 | None = ...,
        substrait_transformation: global___SubstraitTransformationV2 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["substrait_transformation", b"substrait_transformation", "transformation", b"transformation", "user_defined_function", b"user_defined_function"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["substrait_transformation", b"substrait_transformation", "transformation", b"transformation", "user_defined_function", b"user_defined_function"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["transformation", b"transformation"]) -> typing_extensions.Literal["user_defined_function", "substrait_transformation"] | None: ...

global___FeatureTransformationV2 = FeatureTransformationV2

class SubstraitTransformationV2(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUBSTRAIT_PLAN_FIELD_NUMBER: builtins.int
    IBIS_FUNCTION_FIELD_NUMBER: builtins.int
    substrait_plan: builtins.bytes
    ibis_function: builtins.bytes
    def __init__(
        self,
        *,
        substrait_plan: builtins.bytes = ...,
        ibis_function: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ibis_function", b"ibis_function", "substrait_plan", b"substrait_plan"]) -> None: ...

global___SubstraitTransformationV2 = SubstraitTransformationV2



import logging
from functools import wraps

import requests

from feast import RepoConfig
from feast.errors import FeastError
from feast.permissions.client.http_auth_requests_wrapper import (
    get_http_auth_requests_session,
)

logger = logging.getLogger(__name__)


def rest_error_handling_decorator(func):
    @wraps(func)
    def wrapper(config: RepoConfig, *args, **kwargs):
        assert isinstance(config, RepoConfig)

        # Get a Session object
        with get_http_auth_requests_session(config.auth_config) as session:
            # Define a wrapper for session methods
            def method_wrapper(method_name):
                original_method = getattr(session, method_name)

                @wraps(original_method)
                def wrapped_method(*args, **kwargs):
                    logger.debug(
                        f"Calling {method_name} with args: {args}, kwargs: {kwargs}"
                    )
                    response = original_method(*args, **kwargs)
                    logger.debug(
                        f"{method_name} response status code: {response.status_code}"
                    )

                    try:
                        response.raise_for_status()
                    except requests.RequestException:
                        logger.debug(f"response.json() = {response.json()}")
                        mapped_error = FeastError.from_error_detail(response.json())
                        logger.debug(f"mapped_error = {str(mapped_error)}")
                        if mapped_error is not None:
                            raise mapped_error
                    return response

                return wrapped_method

            # Enhance session methods
            session.get = method_wrapper("get")  # type: ignore[method-assign]
            session.post = method_wrapper("post")  # type: ignore[method-assign]
            session.put = method_wrapper("put")  # type: ignore[method-assign]
            session.delete = method_wrapper("delete")  # type: ignore[method-assign]

            # Pass the enhanced session object to the decorated function
            return func(session, config, *args, **kwargs)

    return wrapper



from typing import List, Set

from feast.infra.compute_engines.dag.node import DAGNode


def topological_sort(root: DAGNode) -> List[DAGNode]:
    """
    Topologically sort a DAG starting from a single root node.

    Args:
        root: The root DAGNode.

    Returns:
        A list of DAGNodes in topological order (dependencies first).
    """
    return topological_sort_multiple([root])


def topological_sort_multiple(roots: List[DAGNode]) -> List[DAGNode]:
    """
    Topologically sort a DAG with multiple roots.

    Args:
        roots: List of root DAGNodes.

    Returns:
        A list of all reachable DAGNodes in execution-safe order.
    """
    visited: Set[int] = set()
    ordered: List[DAGNode] = []

    def dfs(node: DAGNode):
        if id(node) in visited:
            return
        visited.add(id(node))
        for input_node in node.inputs:
            dfs(input_node)
        ordered.append(node)

    for root in roots:
        dfs(root)

    return ordered



from typing import Dict, Iterable, Literal, Optional

import pandas as pd
import pyarrow
import pyarrow as pa
from pyspark import SparkConf
from pyspark.sql import SparkSession

from feast.infra.common.serde import SerializedArtifacts
from feast.utils import _convert_arrow_to_proto, _run_pyarrow_field_mapping


def get_or_create_new_spark_session(
    spark_config: Optional[Dict[str, str]] = None,
) -> SparkSession:
    spark_session = SparkSession.getActiveSession()
    if not spark_session:
        spark_builder = SparkSession.builder
        if spark_config:
            spark_builder = spark_builder.config(
                conf=SparkConf().setAll([(k, v) for k, v in spark_config.items()])
            )

        spark_session = spark_builder.getOrCreate()
    spark_session.conf.set("spark.sql.execution.arrow.pyspark.enabled", "true")
    return spark_session


def map_in_arrow(
    iterator: Iterable[pa.RecordBatch],
    serialized_artifacts: "SerializedArtifacts",
    mode: Literal["online", "offline"] = "online",
):
    for batch in iterator:
        table = pa.Table.from_batches([batch])

        (
            feature_view,
            online_store,
            offline_store,
            repo_config,
        ) = serialized_artifacts.unserialize()

        if mode == "online":
            join_key_to_value_type = {
                entity.name: entity.dtype.to_value_type()
                for entity in feature_view.entity_columns
            }

            rows_to_write = _convert_arrow_to_proto(
                table, feature_view, join_key_to_value_type
            )

            online_store.online_write_batch(
                config=repo_config,
                table=feature_view,
                data=rows_to_write,
                progress=lambda x: None,
            )
        if mode == "offline":
            offline_store.offline_write_batch(
                config=repo_config,
                feature_view=feature_view,
                table=table,
                progress=lambda x: None,
            )

        yield batch


def map_in_pandas(iterator, serialized_artifacts: SerializedArtifacts):
    for pdf in iterator:
        if pdf.shape[0] == 0:
            print("Skipping")
            return

        table = pyarrow.Table.from_pandas(pdf)

        (
            feature_view,
            online_store,
            _,
            repo_config,
        ) = serialized_artifacts.unserialize()

        if feature_view.batch_source.field_mapping is not None:
            # Spark offline store does the field mapping in pull_latest_from_table_or_query() call
            # This may be needed in future if this materialization engine supports other offline stores
            table = _run_pyarrow_field_mapping(
                table, feature_view.batch_source.field_mapping
            )

        join_key_to_value_type = {
            entity.name: entity.dtype.to_value_type()
            for entity in feature_view.entity_columns
        }

        rows_to_write = _convert_arrow_to_proto(
            table, feature_view, join_key_to_value_type
        )
        online_store.online_write_batch(
            repo_config,
            feature_view,
            rows_to_write,
            lambda x: None,
        )

    yield pd.DataFrame(
        [pd.Series(range(1, 2))]
    )  # dummy result because mapInPandas needs to return something



import grpc

from feast.errors import FeastError


def exception_wrapper(behavior, request, context):
    try:
        return behavior(request, context)
    except grpc.RpcError as e:
        context.abort(e.code(), e.details())
    except FeastError as e:
        context.abort(
            e.grpc_status_code(),
            e.to_error_detail(),
        )


class ErrorInterceptor(grpc.ServerInterceptor):
    def intercept_service(self, continuation, handler_call_details):
        handler = continuation(handler_call_details)
        if handler is None:
            return None

        if handler.unary_unary:
            return grpc.unary_unary_rpc_method_handler(
                lambda req, ctx: exception_wrapper(handler.unary_unary, req, ctx),
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )
        elif handler.unary_stream:
            return grpc.unary_stream_rpc_method_handler(
                lambda req, ctx: exception_wrapper(handler.unary_stream, req, ctx),
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )
        elif handler.stream_unary:
            return grpc.stream_unary_rpc_method_handler(
                lambda req, ctx: exception_wrapper(handler.stream_unary, req, ctx),
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )
        elif handler.stream_stream:
            return grpc.stream_stream_rpc_method_handler(
                lambda req, ctx: exception_wrapper(handler.stream_stream, req, ctx),
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )
        return handler



from fastapi import APIRouter, Depends, Query

from feast.api.registry.rest.codegen_utils import render_feature_code
from feast.api.registry.rest.rest_utils import (
    aggregate_across_projects,
    create_grpc_pagination_params,
    create_grpc_sorting_params,
    get_object_relationships,
    get_pagination_params,
    get_relationships_for_objects,
    get_sorting_params,
    grpc_call,
)
from feast.registry_server import RegistryServer_pb2
from feast.type_map import _convert_value_type_str_to_value_type
from feast.types import from_value_type


def get_feature_router(grpc_handler) -> APIRouter:
    router = APIRouter()

    @router.get("/features")
    def list_features(
        project: str = Query(...),
        feature_view: str = Query(None),
        name: str = Query(None),
        include_relationships: bool = Query(
            False, description="Include relationships for each feature"
        ),
        allow_cache: bool = Query(
            True, description="Allow using cached registry data (default: true)"
        ),
        pagination_params: dict = Depends(get_pagination_params),
        sorting_params: dict = Depends(get_sorting_params),
    ):
        req = RegistryServer_pb2.ListFeaturesRequest(
            project=project,
            feature_view=feature_view or "",
            name=name or "",
            allow_cache=allow_cache,
            pagination=create_grpc_pagination_params(pagination_params),
            sorting=create_grpc_sorting_params(sorting_params),
        )
        response = grpc_call(grpc_handler.ListFeatures, req)
        if "features" not in response:
            response["features"] = []
        if "pagination" not in response:
            response["pagination"] = {}

        if include_relationships:
            features = response.get("features", [])
            relationships = get_relationships_for_objects(
                grpc_handler, features, "feature", project, allow_cache
            )
            response["relationships"] = relationships
        return response

    @router.get("/features/{feature_view}/{name}")
    def get_feature(
        feature_view: str,
        name: str,
        project: str = Query(...),
        include_relationships: bool = Query(
            False, description="Include relationships for this feature"
        ),
        allow_cache: bool = Query(True),
    ):
        req = RegistryServer_pb2.GetFeatureRequest(
            project=project,
            feature_view=feature_view,
            name=name,
            allow_cache=allow_cache,
        )

        response = grpc_call(grpc_handler.GetFeature, req)

        if include_relationships:
            response["relationships"] = get_object_relationships(
                grpc_handler, "feature", name, project, allow_cache
            )

        if response:
            dtype_str = response.get("type") or response.get("dtype")
            value_type_enum = (
                _convert_value_type_str_to_value_type(dtype_str.upper())
                if dtype_str
                else None
            )
            feast_type = from_value_type(value_type_enum) if value_type_enum else None
            dtype = (
                feast_type.__name__
                if feast_type and hasattr(feast_type, "__name__")
                else "String"
            )
            context = dict(
                name=response.get("name", name),
                dtype=dtype,
                description=response.get("description", ""),
                tags=response.get("tags", response.get("labels", {})) or {},
            )
            response["featureDefinition"] = render_feature_code(context)

        return response

    @router.get("/features/all")
    def list_features_all(
        page: int = Query(1, ge=1),
        limit: int = Query(50, ge=1, le=100),
        sort_by: str = Query(None),
        sort_order: str = Query("asc"),
        include_relationships: bool = Query(
            False, description="Include relationships for each feature"
        ),
        allow_cache: bool = Query(
            True, description="Allow using cached registry data (default: true)"
        ),
    ):
        return aggregate_across_projects(
            grpc_handler=grpc_handler,
            list_method=grpc_handler.ListFeatures,
            request_cls=RegistryServer_pb2.ListFeaturesRequest,
            response_key="features",
            object_type="feature",
            include_relationships=include_relationships,
            allow_cache=allow_cache,
            page=page,
            limit=limit,
            sort_by=sort_by,
            sort_order=sort_order,
        )

    return router



"""
MCP (Model Context Protocol) integration for Feast Feature Server.

This module provides MCP support for Feast by integrating with fastapi_mcp
to expose Feast functionality through the Model Context Protocol.
"""

import logging
from typing import Optional

from feast.feature_store import FeatureStore

logger = logging.getLogger(__name__)

try:
    from fastapi_mcp import FastApiMCP

    MCP_AVAILABLE = True
except ImportError:
    logger.warning(
        "fastapi_mcp is not installed. MCP support will be disabled. "
        "Install it with: pip install fastapi_mcp"
    )
    MCP_AVAILABLE = False
    # Create placeholder classes for testing
    FastApiMCP = None


def add_mcp_support_to_app(app, store: FeatureStore, config) -> Optional["FastApiMCP"]:
    """Add MCP support to the FastAPI app if enabled in configuration."""
    if not MCP_AVAILABLE:
        logger.warning("MCP support requested but fastapi_mcp is not available")
        return None

    try:
        # Create MCP server from the FastAPI app
        mcp = FastApiMCP(
            app,
            name=getattr(config, "mcp_server_name", "feast-feature-store"),
            description="Feast Feature Store MCP Server - Access feature store data and operations through MCP",
        )

        # Mount the MCP server to the FastAPI app
        mcp.mount()

        logger.info(
            "MCP support has been enabled for the Feast feature server at /mcp endpoint"
        )
        logger.info(
            f"MCP integration initialized for {getattr(config, 'mcp_server_name', 'feast-feature-store')} "
            f"v{getattr(config, 'mcp_server_version', '1.0.0')}"
        )

        return mcp

    except Exception as e:
        logger.error(f"Failed to initialize MCP integration: {e}")
        return None



from dataclasses import dataclass
from datetime import datetime
from typing import Optional, Union

import pandas as pd

from feast import BatchFeatureView, StreamFeatureView
from feast.infra.registry.registry import Registry


@dataclass
class HistoricalRetrievalTask:
    project: str
    entity_df: Union[pd.DataFrame, str]
    feature_view: Union[BatchFeatureView, StreamFeatureView]
    full_feature_name: bool
    registry: Registry
    start_time: Optional[datetime] = None
    end_time: Optional[datetime] = None



import json
import os
import weakref
from datetime import datetime
from pathlib import Path
from typing import Any, List, Optional, Union

import grpc
from google.protobuf.empty_pb2 import Empty
from google.protobuf.timestamp_pb2 import Timestamp
from pydantic import StrictStr

from feast.base_feature_view import BaseFeatureView
from feast.data_source import DataSource
from feast.entity import Entity
from feast.feature_service import FeatureService
from feast.feature_view import FeatureView
from feast.infra.infra_object import Infra
from feast.infra.registry.base_registry import BaseRegistry
from feast.on_demand_feature_view import OnDemandFeatureView
from feast.permissions.auth_model import AuthConfig, NoAuthConfig
from feast.permissions.client.grpc_client_auth_interceptor import (
    GrpcClientAuthHeaderInterceptor,
)
from feast.permissions.permission import Permission
from feast.project import Project
from feast.project_metadata import ProjectMetadata
from feast.protos.feast.core.Registry_pb2 import Registry as RegistryProto
from feast.protos.feast.registry import RegistryServer_pb2, RegistryServer_pb2_grpc
from feast.repo_config import RegistryConfig
from feast.saved_dataset import SavedDataset, ValidationReference
from feast.stream_feature_view import StreamFeatureView


def extract_base_feature_view(
    any_feature_view: RegistryServer_pb2.AnyFeatureView,
) -> BaseFeatureView:
    feature_view_type = any_feature_view.WhichOneof("any_feature_view")
    if feature_view_type == "feature_view":
        feature_view = FeatureView.from_proto(any_feature_view.feature_view)
    elif feature_view_type == "on_demand_feature_view":
        feature_view = OnDemandFeatureView.from_proto(
            any_feature_view.on_demand_feature_view
        )
    elif feature_view_type == "stream_feature_view":
        feature_view = StreamFeatureView.from_proto(
            any_feature_view.stream_feature_view
        )

    return feature_view


class RemoteRegistryConfig(RegistryConfig):
    registry_type: StrictStr = "remote"
    """ str: Provider name or a class name that implements Registry."""

    path: StrictStr = ""
    """ str: Path to metadata store.
    If registry_type is 'remote', then this is a URL for registry server """

    cert: StrictStr = ""
    """ str: Path to the public certificate when the registry server starts in TLS(SSL) mode. This may be needed if the registry server started with a self-signed certificate, typically this file ends with `*.crt`, `*.cer`, or `*.pem`.
    If registry_type is 'remote', then this configuration is needed to connect to remote registry server in TLS mode. If the remote registry started in non-tls mode then this configuration is not needed."""

    is_tls: bool = False
    """     bool: Set to `True` if you plan to connect to a registry server running in TLS (SSL) mode.
    If you intend to add the public certificate to the trust store instead of passing it via the `cert` parameter, this field must be set to `True`.
    If you are planning to add the public certificate as part of the trust store instead of passing it as a `cert` parameters then setting this field to `true` is mandatory.
    """


class RemoteRegistry(BaseRegistry):
    def __init__(
        self,
        registry_config: Union[RegistryConfig, RemoteRegistryConfig],
        project: str,
        repo_path: Optional[Path],
        auth_config: AuthConfig = NoAuthConfig(),
    ):
        self.auth_config = auth_config
        assert isinstance(registry_config, RemoteRegistryConfig)
        self.channel = self._create_grpc_channel(registry_config)
        weakref.finalize(self, self.channel.close)

        auth_header_interceptor = GrpcClientAuthHeaderInterceptor(auth_config)
        self.channel = grpc.intercept_channel(self.channel, auth_header_interceptor)
        self.stub = RegistryServer_pb2_grpc.RegistryServerStub(self.channel)

    def _create_grpc_channel(self, registry_config):
        assert isinstance(registry_config, RemoteRegistryConfig)
        if registry_config.cert or registry_config.is_tls:
            cafile = os.getenv("SSL_CERT_FILE") or os.getenv("REQUESTS_CA_BUNDLE")
            if not cafile and not registry_config.cert:
                raise EnvironmentError(
                    "SSL_CERT_FILE or REQUESTS_CA_BUNDLE environment variable must be set to use secure TLS or set the cert parameter in feature_Store.yaml file under remote registry configuration."
                )
            with open(
                registry_config.cert if registry_config.cert else cafile, "rb"
            ) as cert_file:
                trusted_certs = cert_file.read()
            tls_credentials = grpc.ssl_channel_credentials(
                root_certificates=trusted_certs
            )
            return grpc.secure_channel(registry_config.path, tls_credentials)
        else:
            # Create an insecure gRPC channel
            return grpc.insecure_channel(registry_config.path)

    def close(self):
        if self.channel:
            self.channel.close()

    def apply_entity(self, entity: Entity, project: str, commit: bool = True):
        request = RegistryServer_pb2.ApplyEntityRequest(
            entity=entity.to_proto(), project=project, commit=commit
        )
        self.stub.ApplyEntity(request)

    def delete_entity(self, name: str, project: str, commit: bool = True):
        request = RegistryServer_pb2.DeleteEntityRequest(
            name=name, project=project, commit=commit
        )
        self.stub.DeleteEntity(request)

    def get_entity(self, name: str, project: str, allow_cache: bool = False) -> Entity:
        request = RegistryServer_pb2.GetEntityRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetEntity(request)
        return Entity.from_proto(response)

    def list_entities(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[Entity]:
        request = RegistryServer_pb2.ListEntitiesRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListEntities(request)
        return [Entity.from_proto(entity) for entity in response.entities]

    def apply_data_source(
        self, data_source: DataSource, project: str, commit: bool = True
    ):
        request = RegistryServer_pb2.ApplyDataSourceRequest(
            data_source=data_source.to_proto(), project=project, commit=commit
        )
        self.stub.ApplyDataSource(request)

    def delete_data_source(self, name: str, project: str, commit: bool = True):
        request = RegistryServer_pb2.DeleteDataSourceRequest(
            name=name, project=project, commit=commit
        )
        self.stub.DeleteDataSource(request)

    def get_data_source(
        self, name: str, project: str, allow_cache: bool = False
    ) -> DataSource:
        request = RegistryServer_pb2.GetDataSourceRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetDataSource(request)
        return DataSource.from_proto(response)

    def list_data_sources(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[DataSource]:
        request = RegistryServer_pb2.ListDataSourcesRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListDataSources(request)
        return [
            DataSource.from_proto(data_source) for data_source in response.data_sources
        ]

    def apply_feature_service(
        self, feature_service: FeatureService, project: str, commit: bool = True
    ):
        request = RegistryServer_pb2.ApplyFeatureServiceRequest(
            feature_service=feature_service.to_proto(), project=project, commit=commit
        )
        self.stub.ApplyFeatureService(request)

    def delete_feature_service(self, name: str, project: str, commit: bool = True):
        request = RegistryServer_pb2.DeleteFeatureServiceRequest(
            name=name, project=project, commit=commit
        )
        self.stub.DeleteFeatureService(request)

    def get_feature_service(
        self, name: str, project: str, allow_cache: bool = False
    ) -> FeatureService:
        request = RegistryServer_pb2.GetFeatureServiceRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetFeatureService(request)
        return FeatureService.from_proto(response)

    def list_feature_services(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[FeatureService]:
        request = RegistryServer_pb2.ListFeatureServicesRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListFeatureServices(request)
        return [
            FeatureService.from_proto(feature_service)
            for feature_service in response.feature_services
        ]

    def apply_feature_view(
        self, feature_view: BaseFeatureView, project: str, commit: bool = True
    ):
        if isinstance(feature_view, StreamFeatureView):
            arg_name = "stream_feature_view"
        elif isinstance(feature_view, FeatureView):
            arg_name = "feature_view"
        elif isinstance(feature_view, OnDemandFeatureView):
            arg_name = "on_demand_feature_view"

        request = RegistryServer_pb2.ApplyFeatureViewRequest(
            feature_view=(
                feature_view.to_proto() if arg_name == "feature_view" else None
            ),
            stream_feature_view=(
                feature_view.to_proto() if arg_name == "stream_feature_view" else None
            ),
            on_demand_feature_view=(
                feature_view.to_proto()
                if arg_name == "on_demand_feature_view"
                else None
            ),
            project=project,
            commit=commit,
        )

        self.stub.ApplyFeatureView(request)

    def delete_feature_view(self, name: str, project: str, commit: bool = True):
        request = RegistryServer_pb2.DeleteFeatureViewRequest(
            name=name, project=project, commit=commit
        )
        self.stub.DeleteFeatureView(request)

    def get_stream_feature_view(
        self, name: str, project: str, allow_cache: bool = False
    ) -> StreamFeatureView:
        request = RegistryServer_pb2.GetStreamFeatureViewRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetStreamFeatureView(request)
        return StreamFeatureView.from_proto(response)

    def list_stream_feature_views(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[StreamFeatureView]:
        request = RegistryServer_pb2.ListStreamFeatureViewsRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListStreamFeatureViews(request)
        return [
            StreamFeatureView.from_proto(stream_feature_view)
            for stream_feature_view in response.stream_feature_views
        ]

    def get_on_demand_feature_view(
        self, name: str, project: str, allow_cache: bool = False
    ) -> OnDemandFeatureView:
        request = RegistryServer_pb2.GetOnDemandFeatureViewRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetOnDemandFeatureView(request)
        return OnDemandFeatureView.from_proto(response)

    def list_on_demand_feature_views(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[OnDemandFeatureView]:
        request = RegistryServer_pb2.ListOnDemandFeatureViewsRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListOnDemandFeatureViews(request)
        return [
            OnDemandFeatureView.from_proto(on_demand_feature_view)
            for on_demand_feature_view in response.on_demand_feature_views
        ]

    def get_any_feature_view(
        self, name: str, project: str, allow_cache: bool = False
    ) -> BaseFeatureView:
        request = RegistryServer_pb2.GetAnyFeatureViewRequest(
            name=name, project=project, allow_cache=allow_cache
        )

        response: RegistryServer_pb2.GetAnyFeatureViewResponse = (
            self.stub.GetAnyFeatureView(request)
        )
        any_feature_view = response.any_feature_view
        return extract_base_feature_view(any_feature_view)

    def list_all_feature_views(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[BaseFeatureView]:
        request = RegistryServer_pb2.ListAllFeatureViewsRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )

        response: RegistryServer_pb2.ListAllFeatureViewsResponse = (
            self.stub.ListAllFeatureViews(request)
        )
        return [
            extract_base_feature_view(any_feature_view)
            for any_feature_view in response.feature_views
        ]

    def get_feature_view(
        self, name: str, project: str, allow_cache: bool = False
    ) -> FeatureView:
        request = RegistryServer_pb2.GetFeatureViewRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetFeatureView(request)
        return FeatureView.from_proto(response)

    def list_feature_views(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[FeatureView]:
        request = RegistryServer_pb2.ListFeatureViewsRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListFeatureViews(request)

        return [
            FeatureView.from_proto(feature_view)
            for feature_view in response.feature_views
        ]

    def apply_materialization(
        self,
        feature_view: Union[FeatureView, OnDemandFeatureView],
        project: str,
        start_date: datetime,
        end_date: datetime,
        commit: bool = True,
    ):
        start_date_timestamp = Timestamp()
        end_date_timestamp = Timestamp()
        start_date_timestamp.FromDatetime(start_date)
        end_date_timestamp.FromDatetime(end_date)

        # TODO: for this to work for stream feature views, ApplyMaterializationRequest needs to be updated
        request = RegistryServer_pb2.ApplyMaterializationRequest(
            feature_view=feature_view.to_proto(),
            project=project,
            start_date=start_date_timestamp,
            end_date=end_date_timestamp,
            commit=commit,
        )
        self.stub.ApplyMaterialization(request)

    def apply_saved_dataset(
        self,
        saved_dataset: SavedDataset,
        project: str,
        commit: bool = True,
    ):
        request = RegistryServer_pb2.ApplySavedDatasetRequest(
            saved_dataset=saved_dataset.to_proto(), project=project, commit=commit
        )
        self.stub.ApplyFeatureService(request)

    def delete_saved_dataset(self, name: str, project: str, commit: bool = True):
        request = RegistryServer_pb2.DeleteSavedDatasetRequest(
            name=name, project=project, commit=commit
        )
        self.stub.DeleteSavedDataset(request)

    def get_saved_dataset(
        self, name: str, project: str, allow_cache: bool = False
    ) -> SavedDataset:
        request = RegistryServer_pb2.GetSavedDatasetRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetSavedDataset(request)
        return SavedDataset.from_proto(response)

    def list_saved_datasets(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[SavedDataset]:
        request = RegistryServer_pb2.ListSavedDatasetsRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListSavedDatasets(request)
        return [
            SavedDataset.from_proto(saved_dataset)
            for saved_dataset in response.saved_datasets
        ]

    def apply_validation_reference(
        self,
        validation_reference: ValidationReference,
        project: str,
        commit: bool = True,
    ):
        request = RegistryServer_pb2.ApplyValidationReferenceRequest(
            validation_reference=validation_reference.to_proto(),
            project=project,
            commit=commit,
        )
        self.stub.ApplyValidationReference(request)

    def delete_validation_reference(self, name: str, project: str, commit: bool = True):
        request = RegistryServer_pb2.DeleteValidationReferenceRequest(
            name=name, project=project, commit=commit
        )
        self.stub.DeleteValidationReference(request)

    def get_validation_reference(
        self, name: str, project: str, allow_cache: bool = False
    ) -> ValidationReference:
        request = RegistryServer_pb2.GetValidationReferenceRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetValidationReference(request)
        return ValidationReference.from_proto(response)

    def list_validation_references(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[ValidationReference]:
        request = RegistryServer_pb2.ListValidationReferencesRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListValidationReferences(request)
        return [
            ValidationReference.from_proto(validation_reference)
            for validation_reference in response.validation_references
        ]

    def list_project_metadata(
        self, project: str, allow_cache: bool = False
    ) -> List[ProjectMetadata]:
        request = RegistryServer_pb2.ListProjectMetadataRequest(
            project=project, allow_cache=allow_cache
        )
        response = self.stub.ListProjectMetadata(request)
        return [ProjectMetadata.from_proto(pm) for pm in response.project_metadata]

    def update_infra(self, infra: Infra, project: str, commit: bool = True):
        request = RegistryServer_pb2.UpdateInfraRequest(
            infra=infra.to_proto(), project=project, commit=commit
        )
        self.stub.UpdateInfra(request)

    def get_infra(self, project: str, allow_cache: bool = False) -> Infra:
        request = RegistryServer_pb2.GetInfraRequest(
            project=project, allow_cache=allow_cache
        )
        response = self.stub.GetInfra(request)
        return Infra.from_proto(response)

    def apply_user_metadata(
        self,
        project: str,
        feature_view: BaseFeatureView,
        metadata_bytes: Optional[bytes],
    ):
        pass

    def get_user_metadata(
        self, project: str, feature_view: BaseFeatureView
    ) -> Optional[bytes]:
        pass

    def apply_permission(
        self, permission: Permission, project: str, commit: bool = True
    ):
        permission_proto = permission.to_proto()
        permission_proto.spec.project = project

        request = RegistryServer_pb2.ApplyPermissionRequest(
            permission=permission_proto, project=project, commit=commit
        )
        self.stub.ApplyPermission(request)

    def delete_permission(self, name: str, project: str, commit: bool = True):
        request = RegistryServer_pb2.DeletePermissionRequest(
            name=name, project=project, commit=commit
        )
        self.stub.DeletePermission(request)

    def get_permission(
        self, name: str, project: str, allow_cache: bool = False
    ) -> Permission:
        request = RegistryServer_pb2.GetPermissionRequest(
            name=name, project=project, allow_cache=allow_cache
        )
        response = self.stub.GetPermission(request)

        return Permission.from_proto(response)

    def list_permissions(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[Permission]:
        request = RegistryServer_pb2.ListPermissionsRequest(
            project=project, allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListPermissions(request)
        return [
            Permission.from_proto(permission) for permission in response.permissions
        ]

    def apply_project(
        self,
        project: Project,
        commit: bool = True,
    ):
        project_proto = project.to_proto()

        request = RegistryServer_pb2.ApplyProjectRequest(
            project=project_proto, commit=commit
        )
        self.stub.ApplyProject(request)

    def delete_project(
        self,
        name: str,
        commit: bool = True,
    ):
        request = RegistryServer_pb2.DeleteProjectRequest(name=name, commit=commit)
        self.stub.DeleteProject(request)

    def get_project(
        self,
        name: str,
        allow_cache: bool = False,
    ) -> Project:
        request = RegistryServer_pb2.GetProjectRequest(
            name=name, allow_cache=allow_cache
        )
        response = self.stub.GetProject(request)

        return Project.from_proto(response)

    def list_projects(
        self,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[Project]:
        request = RegistryServer_pb2.ListProjectsRequest(
            allow_cache=allow_cache, tags=tags
        )
        response = self.stub.ListProjects(request)
        return [Project.from_proto(project) for project in response.projects]

    def get_project_metadata(self, project: str, key: str) -> Optional[str]:
        request = RegistryServer_pb2.ListProjectMetadataRequest(project=project)
        response = self.stub.ListProjectMetadata(request)
        for pm in response.project_metadata:
            if hasattr(pm, "custom_metadata") and key in pm.custom_metadata:
                return pm.custom_metadata[key]
            try:
                meta = json.loads(pm.project_uuid) if pm.project_uuid else {}
            except Exception:
                meta = {}
            if isinstance(meta, dict) and key in meta:
                return meta[key]
        return None

    def proto(self) -> RegistryProto:
        return self.stub.Proto(Empty())

    def commit(self):
        self.stub.Commit(Empty())

    def refresh(self, project: Optional[str] = None):
        request = RegistryServer_pb2.RefreshRequest(project=str(project))
        self.stub.Refresh(request)

    # Lineage operations
    def get_registry_lineage(
        self,
        project: str,
        allow_cache: bool = False,
        filter_object_type: Optional[str] = None,
        filter_object_name: Optional[str] = None,
    ) -> tuple[List[Any], List[Any]]:
        """Get complete registry lineage via remote registry server."""
        request = RegistryServer_pb2.GetRegistryLineageRequest(
            project=project,
            allow_cache=allow_cache,
            filter_object_type=filter_object_type or "",
            filter_object_name=filter_object_name or "",
        )
        response = self.stub.GetRegistryLineage(request)

        # Convert protobuf responses back to lineage objects
        from feast.lineage.registry_lineage import (
            EntityReference,
            EntityRelation,
            FeastObjectType,
        )

        relationships = []
        for rel_proto in response.relationships:
            relationships.append(
                EntityRelation(
                    source=EntityReference(
                        FeastObjectType(rel_proto.source.type), rel_proto.source.name
                    ),
                    target=EntityReference(
                        FeastObjectType(rel_proto.target.type), rel_proto.target.name
                    ),
                )
            )

        indirect_relationships = []
        for rel_proto in response.indirect_relationships:
            indirect_relationships.append(
                EntityRelation(
                    source=EntityReference(
                        FeastObjectType(rel_proto.source.type), rel_proto.source.name
                    ),
                    target=EntityReference(
                        FeastObjectType(rel_proto.target.type), rel_proto.target.name
                    ),
                )
            )

        return relationships, indirect_relationships

    def get_object_relationships(
        self,
        project: str,
        object_type: str,
        object_name: str,
        include_indirect: bool = False,
        allow_cache: bool = False,
    ) -> List[Any]:
        """Get relationships for a specific object via remote registry server."""
        request = RegistryServer_pb2.GetObjectRelationshipsRequest(
            project=project,
            object_type=object_type,
            object_name=object_name,
            include_indirect=include_indirect,
            allow_cache=allow_cache,
        )
        response = self.stub.GetObjectRelationships(request)

        # Convert protobuf responses back to lineage objects
        from feast.lineage.registry_lineage import (
            EntityReference,
            EntityRelation,
            FeastObjectType,
        )

        relationships = []
        for rel_proto in response.relationships:
            relationships.append(
                EntityRelation(
                    source=EntityReference(
                        FeastObjectType(rel_proto.source.type), rel_proto.source.name
                    ),
                    target=EntityReference(
                        FeastObjectType(rel_proto.target.type), rel_proto.target.name
                    ),
                )
            )

        return relationships

    def teardown(self):
        pass
